//
// GetEnvNode.java	Version 1.0	9/8/2004
//              
// Licensed Materials - Property of IBM
//
// 5648-C63
// (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
//

/* GetEnvNode:
 *		The GetEnvNode supports three Environment Variable groups:
 *			System -- the os environment variables
 *			Java -- the Java runtime properties
 *			Message Broker -- Message Broker runtime environment variables
 *
 *		The Environment Variable groups are node properties.  The
 *		variables associated with each group are written to sub-trees in the 
 *		Environment syntax tree, as shown:
 *			System -- Environment.ENV
 *			Java   -- Environment.ENV.JAVA
 *			Message Broker -- Environment.ENV.MB
 *
 *		The node is initialized once, where a Properties object is created for
 *		each of the selected Environment Variable groups.
 *
 *		When evaluate() is called, the sub-trees in the Environment syntax 
 *		tree are built from the Environment Properties objects 
 */	 

package com.ibm.jpluginx;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Properties;

import com.ibm.broker.plugin.MbElement;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbInputTerminal;
import com.ibm.broker.plugin.MbMessageAssembly;
import com.ibm.broker.plugin.MbNode;
import com.ibm.broker.plugin.MbNodeInterface;
import com.ibm.broker.plugin.MbOutputTerminal;
import com.ibm.broker.plugin.MbService;
import com.ibm.broker.plugin.MbUserException;

public class GetEnvNode extends MbNode implements MbNodeInterface
{
	// variables for node's exposed properties
	// Basic Group
	private boolean useSysVars;
	private boolean useJavaVars;
	private boolean useMBVars;
	private int convert;
	// Trace Group
	private boolean isTrace;
	private String traceFile;
	
	// variables for trace output to file 
	private SimpleDateFormat timeFormat; 
	private String newLine;
	private boolean traceToFile;
	private FileWriter fWriter = null;
	
	// variables to cache the three Enviroment Variable groups
	// System, Java and Message Broker
	private Properties sysVars;
	private Properties javaVars;
	private Properties mbVars;
	
	// variables for message broker environment varialbes.
	// Note: ThreadName and ElaspedTime are extentions
	private String msgFlowStr;
	private String executionGrpStr;
	private String brokerStr;
	private String qMgrStr;
	private String dataSrcUserIdStr;
	private String elapsedTimeElName;
	private boolean isCoordinatedTx;
	private String threadElName;
	
	// constants for Convert states
	private static final int NOCONVERT = 0;
	private static final int UPCONVERT = 1;
	private static final int LOCONVERT = 2;
	
	// constants for trace output
	private static final int TRACEENTER = 0;
	private static final int TRACEEXIT = 1;
	private static final int TRACENONE = 2;
	private static final String[] TRACEACTIONSTR = {">>ENTER ","<<EXIT  ",""};
	private static final String[] TRACETABSTR = {"","    ","        ","            ","                ","                    "};
	private static final String TRACEGETENVSTR = "GetEnvNode:  ";
	private static final String TRACEFUNCPREFIXSTR = "():  ";
	
	// the node's name
	private static final String getEnvNodeName = "GetEnvNode";
	
	// node initialization
	private boolean isInitialized;

    // Return the name of the GetEnv node
	public static String getNodeName()
	{
		System.out.println(
			"GetEnvNode:  getNodeName(),  returning <" + getEnvNodeName + ">");
		return getEnvNodeName;
	}
	

	public GetEnvNode() throws MbException
	{
		// create the input and output terminals
		createInputTerminal("In");
		createOutputTerminal("Failure");
		createOutputTerminal("Out");
		
		// initialize property variables
		isTrace = false;
		traceFile = null;
		traceToFile = false;
		newLine = null;
		sysVars = null;
		javaVars = null;
		mbVars = null;
		useSysVars = false;
		useJavaVars = false;
		useMBVars = false;
		convert = NOCONVERT;
		
		// initialize Message Broker variables
		msgFlowStr = null;
		executionGrpStr = null;
		brokerStr = null;
		qMgrStr = null;
		dataSrcUserIdStr = null;
		threadElName = null;
		elapsedTimeElName = null;
		isCoordinatedTx = false;
		
		// node is not fully initialized yet
		isInitialized = false;
	}
	
	
	// Broker will call this method to perform node cleanup
	public void onDelete() throws MbException
	{
		if ((traceToFile == true) && (fWriter != null))
		{
			try
			{
				fWriter.close();
			}
			catch (IOException e)
			{
				// not worth throwing an exception at this point
				if (isTrace)
				{
					// assuming that the file can still be written to
					traceNode(TRACENONE,0,"onDelete","failed to close trace file <" + traceFile + ">");
				}
			}
		}
	}
	

	// Write a GetEnvNode specific entry to the WBIMB log
	private void logWMQI(String method, String key, String entry)
		throws MbException
	{
		// E1, E2, E3, E4, E5, E6 supported -- 
		// E1, E2, E3 pass a null entry
		// E4 passes either "SYS", "JAVA" or "MB"
		// E5 and E6 pass the name of the trace file
		String[] inserts = null;

		if (entry == null)
		{
			inserts = new String[3];
			inserts[0] = msgFlowStr;
			inserts[1] = executionGrpStr;
			inserts[2] = brokerStr;
		}
		else
		{
			inserts = new String[4];
			inserts[0] = entry;
			inserts[1] = msgFlowStr;
			inserts[2] = executionGrpStr;
			inserts[3] = brokerStr;
		}

		MbService.logError(
			this,
			method,
			"com.ibm.jpluginx.GetEnvNodeMessages",
			key,
			"",
			inserts);
	}


    // Throw a GetEnvNode specific WBIMB Exception
	private void throwWMQI(String method, String entry) throws MbException
	{
		String[] inserts = new String[1];
		inserts[0] = entry;
		// Use WBIMB 5.0 message catalog
		MbException userException =
			new MbUserException(this, method, "BIPv500", "4362", "", inserts);
		throw userException;
	}



	private void traceNode(int Action, int Level, String Function, String TraceMsg) throws MbException
	{
		Date currentTime = new Date();
		String outStr = "[" + timeFormat.format(currentTime) + "]  " + 
						TRACETABSTR[Level] +
						TRACEACTIONSTR[Action] + 
						TRACEGETENVSTR +
						Function +
						TRACEFUNCPREFIXSTR +
						TraceMsg;
						
		if (traceToFile == false)
		{
			// trace to stdout
			System.out.println(outStr);
		}
		else
		{
			// trace to file
			try
			{
				fWriter.write(outStr + newLine);
			}
			catch (IOException e)
			{
				// Don't try and trace this error, as there is a problem writing 
				// to the trace file
				logWMQI("initNode", "E5", traceFile);
				throwWMQI("initNode",
						  "Could not write to trace file <" + traceFile + ">");
			}
		}
	}

    

    // Node initialization routine
	private synchronized void initNode() throws MbException
	{	
		if (isInitialized == false)
		{
			// initialize Broker vars first, as they are used if node 
			// initialization fails
			brokerStr = getBroker().getName();
			qMgrStr = getBroker().getQueueManagerName();
			dataSrcUserIdStr = getBroker().getDataSourceUserId();
			msgFlowStr = getMessageFlow().getName();
			executionGrpStr = getExecutionGroup().getName();
			isCoordinatedTx = getMessageFlow().isCoordinatedTransaction();
			
			// Format for DateTime stamp, when trace output is to file
			timeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS z",Locale.getDefault());
			// get platform dependent newline, when trace output is to file
			newLine = System.getProperty("line.separator");
			
			if ((isTrace == true) && (traceFile.compareToIgnoreCase("") != 0))
			{
				// open the trace output file
				try
				{
					// preserve trace file contents if file already exists
					fWriter = new FileWriter(traceFile,true);
					traceToFile = true;
				}
				catch (IOException e)
				{
					// Don't trace this error, as trace file couldn't be opened
					logWMQI("initNode", "E6", traceFile);
					throwWMQI(
						"initNode",
						"Could not open trace file <" + traceFile + ">,  node not initialized");
				}
			}
			
			if (isTrace)
			{
				// can't trace 1st invocation of evaluate until node tracing has
				// been initialized
				traceNode(TRACEENTER,0,"evaluate","");
				traceNode(TRACEENTER,2,"initNode","");
			}

			Process p = null;
			sysVars = new Properties();
			Runtime r = Runtime.getRuntime();
			String OS = System.getProperty("os.name").toLowerCase();
		
			

            // setup for SYSTEM Variables
			if (useSysVars == true)
			{
				try
				{
					if (isTrace)
					{
						traceNode(TRACENONE,3,"initNode","createing SYSTEM vars for OS<" + OS + ">");
					}

 					// On Windows platforms the 'set' command is used to retrieve 
 					// the System Variables
					if ((OS.indexOf("nt") > -1)
						|| (OS.indexOf("windows 2000") > -1)
						|| (OS.indexOf("windows xp") > -1)
						|| (OS.indexOf("win") > -1))
					{
						p = r.exec("cmd.exe /c set");
					}
					else // assume all else are UNIX
					{
						/* On all other platforms (UNIX), the 'env' command is used
						 * to retrieve the System Variables
						 * 
						 * SOLARIS = "SunOS"
						 * LINUX = "Linux"
						 * AIX = 'AIX'
						 * HP-UX = 'HP-UX'
						 */

						p = r.exec("env");
					}

					if (p != null)
					{
						// read the return of the set/env command, line by line
						// and build the associated Properties structure
						BufferedReader br =
							new BufferedReader(
								new InputStreamReader(p.getInputStream()));
						String line, key, value;
						int idx;
						while ((line = br.readLine()) != null)
						{
							idx = line.indexOf('=');
							if (convert == UPCONVERT)
							{
								key = line.substring(0, idx).toUpperCase();
							}
							else if (convert == LOCONVERT)
							{
								key = line.substring(0, idx).toLowerCase();
							}
							else
							{
								key = line.substring(0, idx);
							}
							value = line.substring(idx + 1);
							sysVars.setProperty(key, value);

							if (isTrace)
							{
								traceNode(TRACENONE,4,"initNode","created SYSTEM,  Key=<"
										+ key
										+ ">,  Value=<"
										+ value
										+ ">");
							}
						}
					}
				}
				catch (IOException e)
				{
					traceNode(TRACENONE,4,"initNode","IO Exception reading SYSTEM variables, node not initialized.");
					logWMQI("initNode", "E1", null);
					throwWMQI("initNode",
						      "Could not read SYSTEM variables, node not initialized.");
				}
			}
			
			// ElapsedTime is an extention to the Environment.ENV sub-tree
			elapsedTimeElName = "ElapsedTime";
			if (convert == UPCONVERT)
			{
				elapsedTimeElName = elapsedTimeElName.toUpperCase();
			}
			else if (convert == LOCONVERT)
			{
				elapsedTimeElName = elapsedTimeElName.toLowerCase();
			}
			

			// setup for JAVA Variables
			if (useJavaVars == true)
			{
				java.util.Properties jp = System.getProperties();
				java.util.Enumeration keys = jp.keys();
				javaVars = new Properties();
				Enumeration values = jp.elements();
				String k, v;

				if (isTrace)
				{
					traceNode(TRACENONE,3,"initNode","creating JAVA vars");
				}

				// iterate over the JAVA properties
				while (keys.hasMoreElements())
				{
					v = (String) values.nextElement();

					// JAVA Properties use '.' as name separator.  This will be 
					// inconvienient to work with in ESQL.  Change the dots to underscores.
					if (convert == UPCONVERT)
					{
						k =
							((String) keys.nextElement())
								.replace('.', '_')
								.toUpperCase();
					}
					else if (convert == LOCONVERT)
					{
						k =
							((String) keys.nextElement())
								.replace('.', '_')
								.toLowerCase();
					}
					else
					{
						k = ((String) keys.nextElement()).replace('.', '_');
					}

					// set the properties Key and Value
					javaVars.setProperty(k, v);

					if (isTrace)
					{
						traceNode(TRACENONE,4,"initNode","created JAVA,  Key=<"
								+ k
								+ ">,  Value=<"
								+ v
								+ ">");
					}
				}
			}

			// Setup Message Broker variables
			if (useMBVars == true)
			{
				if (isTrace)
				{
					traceNode(TRACENONE,3,"initNode","creating MB vars");
				}

				mbVars = new Properties();
				
				String brokerElName = "BrokerName";
				String qMgrElName = "QueueManagerName";
				String dataSrcUserIdElName = "DataSourceUserId";
				String executionGroupElName = "ExecutionGroupName";
				String messageFlowElName = "MessageFlowName";
				String coordinatedTxElName = "CoordinatedTransaction";
				String nodeElName = "NodeName";
				threadElName = "ThreadName"; // this is an extention

				if (convert == UPCONVERT)
				{
					brokerElName = brokerElName.toUpperCase();
					qMgrElName = qMgrElName.toUpperCase(); 
					dataSrcUserIdElName = dataSrcUserIdElName.toUpperCase(); 
					executionGroupElName = executionGroupElName.toUpperCase(); 
					messageFlowElName = messageFlowElName.toUpperCase(); 
					coordinatedTxElName = coordinatedTxElName.toUpperCase(); 
					nodeElName = nodeElName.toUpperCase(); 
					threadElName = threadElName.toUpperCase(); 
				}
				else if (convert == LOCONVERT)
				{
					brokerElName = brokerElName.toLowerCase(); 
					qMgrElName = qMgrElName.toLowerCase(); 
					dataSrcUserIdElName = dataSrcUserIdElName.toLowerCase(); 
					executionGroupElName = executionGroupElName.toLowerCase(); 
					messageFlowElName = messageFlowElName.toLowerCase(); 
					coordinatedTxElName = coordinatedTxElName.toLowerCase(); 
					nodeElName = nodeElName.toLowerCase(); 
					threadElName = threadElName.toLowerCase(); 
				}
				
				mbVars.setProperty(brokerElName, brokerStr);
				if (isTrace)
				{
					traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + brokerElName +
					           ">,  Value=<" + brokerStr + ">");
				}
				
				mbVars.setProperty(qMgrElName, qMgrStr);
				if (isTrace)
				{
					traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + qMgrElName +
					           ">,  Value=<" + qMgrStr + ">");
				}
				
				mbVars.setProperty(dataSrcUserIdElName, dataSrcUserIdStr);
				if (isTrace)
				{
					traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + dataSrcUserIdElName +
					           ">,  Value=<" + dataSrcUserIdStr + ">");
				}
				
				mbVars.setProperty(executionGroupElName, executionGrpStr);
				if (isTrace)
				{
					traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + executionGroupElName +
					           ">,  Value=<" + executionGrpStr + ">");
				}
				
				mbVars.setProperty(messageFlowElName, msgFlowStr);
				if (isTrace)
				{
					traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + messageFlowElName +
					           ">,  Value=<" + msgFlowStr + ">");
				}
				
				if (isCoordinatedTx == true)
				{
					mbVars.setProperty(coordinatedTxElName, "TRUE");
					if (isTrace)
					{
						traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + coordinatedTxElName +
					           ">,  Value=<TRUE>");
					}
				}
				else
				{
					mbVars.setProperty(coordinatedTxElName, "FALSE");
					if (isTrace)
					{
						traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + coordinatedTxElName +
					           ">,  Value=<FALSE>");
					}
				}
				
				mbVars.setProperty(nodeElName, getName());
				if (isTrace)
				{
					traceNode(TRACENONE,4,"initNode","created MB,  Key=<" + nodeElName +
					           ">,  Value=<" + getName() + ">");
				}

			}

			isInitialized = true;
		}

		if (isTrace)
		{
			traceNode(TRACEEXIT,2,"initNode","");
		}
	}

    
    /* Build the Environment Variable Syntax trees 
     *
     * Three sub-trees are supported:
     * 		1. System 			-- 	Environment.ENV
     * 		2. Java				-- 	Environment.ENV.JAVA
     *		3. Message Broker	--  Environment.ENV.MB
     */
	private void buildEnvTrees(MbMessageAssembly msgAss) throws MbException
	{
		if (isTrace)
		{
			traceNode(TRACEENTER,2,"buildEnvTrees","");
		}

		// Create the Environment.ENV element
		MbElement envEnv = null;
		try
		{
			envEnv =
				msgAss
					.getGlobalEnvironment()
					.getRootElement()
					.createElementAsLastChild(
					MbElement.TYPE_NAME,
					"ENV",
					null);
					
			if (isTrace)
			{
				traceNode(TRACENONE,3,"buildEnvTrees","created element Environment.ENV");
			}

		}
		catch (MbException e)
		{
			traceNode(TRACENONE,3,"buildEnvTrees","couldn't create Environment.ENV elememt.");
			logWMQI("buildEnvTrees", "E2", null);
			throwWMQI(
				"buildEnvTrees",
				"Couldn't create Environment.ENV elememt.");
		}


		if (useSysVars == true && sysVars != null)
		{
			// Build the System Variable sub-tree -- Environment.ENV
			buildEnvTree(envEnv, "", sysVars.elements(), sysVars.keys());
		}

		if (useJavaVars == true && javaVars != null)
		{
			// Build the Java Variable sub-tree -- Environment.JAVA
			buildEnvTree(envEnv, "JAVA", javaVars.elements(), javaVars.keys());
		}

		if (useMBVars == true && mbVars != null)
		{
			// Build the Message Broker Variable sub-tree -- Environment.ENV.MB
			buildEnvTree(envEnv, "MB", mbVars.elements(), mbVars.keys());

			try
			{
				// We extend the Message Broker variables with the ThreadName variable
				MbElement el =
					msgAss
						.getGlobalEnvironment()
						.getRootElement()
						.getFirstElementByPath("ENV/MB")
						.createElementAsLastChild(
							MbElement.TYPE_NAME_VALUE,
							threadElName,
							Thread.currentThread().getName());
							
				if (isTrace)
				{
					traceNode(TRACENONE,3,"buildEnvTrees","created element Environment.ENV.MB." + threadElName);
				}
			}
			catch (MbException e)
			{
				traceNode(TRACENONE,3,"buildEnvTrees","couldn't create elment Environment.ENV.MB." + threadElName);
				logWMQI("buildEnvTrees", "E3", null);
				throwWMQI(
					"buildEnvTrees",
					"Error creating element Environment.ENV.MB." + threadElName);
			}
		}

		if (isTrace)
		{
			traceNode(TRACEEXIT,2,"buildEnvTrees","");
		}
	}


	/* Build the supported Environment Variable sub-tree defined by "pathName"
	 * 		ENV = null
	 *      JAVA = "JAVA"
	 *      MB = "MB"
	 */
	private void buildEnvTree(
		MbElement env,
		String pathName,
		Enumeration valEnum,
		Enumeration keyEnum)
		throws MbException
	{
		String k, v;
		MbElement el = null;

		if (isTrace)
		{
			if (pathName.compareToIgnoreCase("") == 0)
			{
				traceNode(TRACEENTER,4,"buildEnvTree","using parent element Environment.ENV");
			}
			else
			{
				traceNode(TRACEENTER,4,"buildEnvTree","using parent element Environment.ENV." +
				          pathName);
			}
		}

		try
		{
			// Create the JAVA or MB Element (Environment.ENV already exists)
			if (pathName.compareToIgnoreCase("") != 0)
			{
				el =
				env.createElementAsLastChild(
					MbElement.TYPE_NAME,
					pathName,
					null);
					
				if (isTrace)
				{
					traceNode(TRACENONE,5,"buildEnvTree","created element Environment.ENV." + pathName);
				}
			}
			else
			{
				el = env;
			}
			

			// For each Element in the Enumeration, create a cooresponding
			// syntax Element in the sub-tree
			while (valEnum.hasMoreElements())
			{
				k = ((String) keyEnum.nextElement());
				v = (String) valEnum.nextElement();

				el.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, k, v);

				if (isTrace)
				{
					traceNode(TRACENONE,5,"buildEnvTree","created element Name=<"
							+ k
							+ ">, Value=<"
							+ v
							+ ">");
				}
			}
		}
		catch (MbException e)
		{
			traceNode(TRACENONE,5,"buildEnvTree","error creating elements for Environment.ENV."
					+ pathName);
			logWMQI("buildEnvTree", "E4", pathName);
			throwWMQI(
				"buildEnvTree",
				"Error creating element Environment.ENV." + pathName);
		}

		if (isTrace)
		{
			traceNode(TRACEEXIT,4,"buildEnvTree","");
		}
	}

	/**
	 * @see com.ibm.broker.plugin.MbNodeInterface#evaluate(MbMessageAssembly, MbInputTerminal)
	 */
	public void evaluate(MbMessageAssembly MsgAss, MbInputTerminal InTerm)
		throws MbException
	{
		// Get the start time, for this invocation, in milliseconds
		long startT, eT;
		startT = System.currentTimeMillis();

		if (isInitialized == false)
		{
			initNode();
		}
		else if (isTrace)
		{
			traceNode(TRACEENTER,0,"evaluate","");
		}

		/* Only one Environment.ENV element is allowed.
		 * If a previous Environment.ENV element exists, remove it
		 * and build a new sub-tree
		 */
		MbElement envElem =
			MsgAss
				.getGlobalEnvironment()
				.getRootElement()
				.getFirstElementByPath(
				"ENV");
		if (envElem != null)
		{
			// start with a clean ENV element
			envElem.detach();
		}

		// Build the Environment Variable sub-trees
		buildEnvTrees(MsgAss);

		// get the output terminal
		MbOutputTerminal outTerm = getOutputTerminal("Out");

		// We extend the System variables with the node's elapsed time -- ElapsedTime
		// Calculate the elapsed time and create the syntax element
		eT = System.currentTimeMillis() - startT;
		MbElement etEl =
			MsgAss
				.getGlobalEnvironment()
				.getRootElement()
				.getFirstElementByPath("ENV")
				.createElementAsLastChild(
					MbElement.TYPE_NAME_VALUE,
					elapsedTimeElName,
					Long.toString(eT));
					
		if (isTrace)
		{
			traceNode(TRACENONE,1,"evaluate","created element Environment.ENV." + elapsedTimeElName);
		}

		// propagate the new message assembly
		if (outTerm != null)
		{
			if (isTrace)
			{
				traceNode(TRACENONE,1,"evaluate","calling propagate()");
			}
			
			outTerm.propagate(MsgAss);

			if (isTrace)
			{
				traceNode(TRACENONE,1,"evaluate","return from propagate()");
			}
		}
		
		if (isTrace)
		{
			traceNode(TRACEEXIT,0,"evaluate","");
			
			if (traceToFile)
			{
				try
				{
					fWriter.flush();
				}
				catch (IOException e)
				{
					// ignore exception on flush()
					// exception is handled on write() in traceNode()
				}
			}
		}

	}


	public void setConvert(String val)
	{
		System.out.println(
			"GetEnvNode:  setConvert(),  >>  ENTER,  value<" + val + ">,  <<  EXIT");

		if (val.equalsIgnoreCase("no"))
		{
			convert = NOCONVERT;
		}
		else if (val.equalsIgnoreCase("upper"))
		{
			convert = UPCONVERT;
		}
		else if (val.equalsIgnoreCase("lower"))
		{
			convert = LOCONVERT;
		}
	}
	

	public String getConvert()
	{		
		String rtnStr = "no";
		if (this.convert == UPCONVERT)
		{
			rtnStr = "upper";
		}
		else if (this.convert == LOCONVERT)
		{
			rtnStr = "lower";
		}
		
		System.out.println(
			"GetEnvNode:  getConvert(),  >>  ENTER,  returning<"
				+ rtnStr
				+ ">,  <<  EXIT");

		return rtnStr;
	}


	public void setSrcSystem(String val)
	{
		System.out.println(
			"GetEnvNode:  setSrcSystem(),  >>  ENTER,  value<"
				+ val
				+ ">,  <<  EXIT");

		if (val.compareToIgnoreCase("true") == 0)
		{
			useSysVars = true;
		}
		else
		{
			useSysVars = false;
		}

	}


	public void setSrcJava(String val)
	{
		System.out.println(
			"GetEnvNode:  setSrcJava(),  >>  ENTER,  value<" + val + ">,  <<  EXIT");

		if (val.compareToIgnoreCase("true") == 0)
		{
			useJavaVars = true;
		}
		else
		{
			useJavaVars = false;
		}
	}


	public void setSrcMb(String val)
	{
		System.out.println(
			"GetEnvNode:  setSrcMb() >>  ENTER,  value<" + val + ">,  <<  EXIT");

		if (val.compareToIgnoreCase("true") == 0)
		{
			useMBVars = true;
		}
		else
		{
			useMBVars = false;
		}
	}


	public void setTrace(String val)
	{
		System.out.println(
			"GetEnvNode:  setTrace(),  >>  ENTER,  value<" + val + ">,  <<  EXIT");

		if (val.compareToIgnoreCase("true") == 0)
		{
			isTrace = true;
		}
		else
		{
			isTrace = false;
		}
	}
	
	
	public void setTraceFile(String val)
	{
		System.out.println(
			"GetEnvNode:  setTraceFile(),  >>  ENTER,  value<" + val + ">,  <<  EXIT");
		traceFile = val;
	}


	public String getSrcSystem()
	{
		System.out.println(
			"GetEnvNode:  getSrcSystem(),  >>  ENTER,  returning<"
				+ this.useSysVars
				+ ">,  <<  EXIT");

		if (useSysVars == true)
		{
			return "true";
		}
		return "false";
	}


	public String getSrcJava()
	{
		System.out.println(
			"GetEnvNode:  getSrcJava(),  >>  ENTER,  returning<"
				+ this.useJavaVars
				+ ">,  <<  EXIT");

		if (useJavaVars == true)
		{
			return "true";
		}
		return "false";
	}


	public String getSrcMb()
	{
		System.out.println(
			"GetEnvNode:  getSrcMb(),  >>  ENTER,  returning<"
				+ this.useMBVars
				+ ">,  <<  EXIT");

		if (useMBVars == true)
		{
			return "true";
		}
		return "false";
	}


	public String getTrace()
	{
		System.out.println(
			"GetEnvNode:  getTrace(),  >>  ENTER,  returning<"
				+ this.isTrace
				+ ">,  <<  EXIT");

		if (isTrace == true)
		{
			return "true";
		}
		return "false";
	}
	
	
	public String getTraceFile()
	{
		System.out.println(
			"GetEnvNode:  getTraceFile(),  >>  ENTER,  returning<"
				+ this.traceFile
				+ ">,  <<  EXIT");
		
		return traceFile;
	}

}
